#include "grafi.h"
#include "coda.h"
#include "pila.h"
#include <iostream>

#define MAX 50


void InitGrafo(Grafo& g, int n_nodi) {
	int i;
	g.n_nodi = n_nodi;
	g.nodi = new int*[n_nodi];
	for(i=0;i<n_nodi;i++)
		g.nodi[i]= new int[n_nodi];
		
	for (i=0; i<g.n_nodi; i++) 
		for (int j=0; j<g.n_nodi; j++) 
			g.nodi[i][j] = MAX_INT;
}

void Connect(Grafo& g, int x, int y) {
	g.nodi[x][y] = 0;
}

void Connect(Grafo& g, int x, int y, int peso) {
	g.nodi[x][y] = peso;
}

void VisitaAmpiezza(Grafo& g, int s) {
	int i;
	int *visitato = new int[g.n_nodi];
	Coda c;
	
	//Inizializzazioni
	InitCoda(c);
	for (i=0; i<g.n_nodi; i++) {
		visitato[i] = 0;	//false
	}
	
	cout << "Nodo iniziale" << s << '\t';
	visitato[s] = 1;
	Insert(c, s);
	
	while(!EmptyCoda(c)) 
    {
		int j;
		Extract(c,j);
		
		for (int k=0; k<g.n_nodi; k++) 
        {
			if (g.nodi[j][k]!=MAX_INT) //se l'arco esiste
            {
				if (!visitato[k]) //ed il nodo non era gi� stato visitato
                {
					cout << k << '\t';  // lo visito
					visitato[k] = 1;	// true
					Insert(c, k);
				}
			}
		}
	}
	cout << endl;
}

void VisitaProfondita(Grafo& g, int s) {
	int *visitato = new int[g.n_nodi];
	Pila p;
	
	//Inizializzazioni
	InitPila(p);
	for (int i=0; i<g.n_nodi; i++) {
		visitato[i] = 0;	//false
	}

	visitato[s] = 1;
	Push(p, s); 
	cout << s << '\t';
	while(!EmptyPila(p)) {
		int j;
		Pop(p,j);

		int t=-1;
		for (int k=0; k<g.n_nodi; k++) {
			if (g.nodi[j][k]!=MAX_INT && !visitato[k]) {
				if (t==-1)
					t = k;
				else 
					Push(p, k);
			}
		}
		
		if (t!=-1 && !visitato[t]) {
			cout << t << '\t';
			visitato[t] = 1;	//true
			Push(p, t);
		}
		if (!visitato[j]) {
			cout <<j << '\t';
			visitato[j] = 1;	//true
		}
		
	}
	cout << endl;
}

void VisitaProfonditaRicor(Grafo& g, int s, int* v) {
	if (v==NULL) {	//prima chiamata... inizializza il vettore dei visitati
		v =new int[g.n_nodi];
		for (int i=0;i<g.n_nodi; i++)
			v[i] = 0;
	}
	
	v[s] = 1;
	cout << s << '\t';
	for (int i=0; i<g.n_nodi;i++) {
		if (g.nodi[s][i]<MAX_INT && !v[i]) 
			VisitaProfonditaRicor(g,i,v);
	}
}

void dump(Grafo& g, int* dist, int* completato, int u, int passo) {
	int i;
	cout << "PASSO : " << passo << endl << "dist:\t";
	for (i = 0; i<g.n_nodi; i++)
		if (dist[i]==MAX_INT)
			cout << "~\t";
		else
			cout << dist[i] << '\t';
	cout << endl << "comp:\t";
	for (i = 0; i<g.n_nodi; i++)
		cout << completato[i] << '\t';
	cout << endl << "u: " << u << endl << "-----------------------" << endl;
}

void Dijkstra(Grafo& g, int s) {
	int i;
	int pax = 0;
	int *dist = new int[g.n_nodi];
	int *completato = new int[g.n_nodi];
	
	//inizializziamo il vettore delle distanze e il vettore di booleani completato
	for (i=0; i<g.n_nodi; i++) {
		dist[i] = MAX_INT;
		completato[i] = 0; //false
	}
	
	//inizializziamo dist[s] a zero
	dist[s] = 0;

	dump(g, dist, completato, -1, pax);
	
	int finito = 0; //false
	while(!finito) {
		pax++;

		/* prendiamo un nodo u tale che dist[u] sia 
                il minimo tra quelli con completato[u]==0 */
		int min = MAX_INT;
		int u = 0;
		//cerco il minimo
		for (i=0; i<g.n_nodi;i++) { 
			if (completato[i]==0 && dist[i]<=min) {
				min = dist[i];
				u = i;
			}
		}
		
		completato[u] = 1;
		
		//se il vettore completato � tutto a true abbiamo finito
		finito = 1; // true
		for (i=0; i<g.n_nodi; i++) 
			if (completato[i]==0)
				finito = 0; //false;
						
		//aggiorniamo le distanze
		for (int j=0; j<g.n_nodi; j++) {
			if (dist[j]>(dist[u]+g.nodi[u][j])) {
				dist[j] = dist[u]+g.nodi[u][j];
			}
		}

		dump(g, dist, completato, u, pax);
	}
	
	//stampa il vettore delle distanze
	cout << endl;
	for (i=0; i<g.n_nodi; i++)
		if (dist[i]==MAX_INT)
			cout << "~\t";
		else
			cout << dist[i] << '\t';
	cout << endl;
}

/* funzione di utilita' */
void StampaGrafo(Grafo& g) {
	for (int i=0; i<g.n_nodi; i++) {
		for (int j=0; j<g.n_nodi; j++) 
			cout << g.nodi[i][j] << '\t';
		cout << endl;
	}
}

/////////////// LEZIONE 23 11 2009 ///////////////////////
/*
**	elimina_vertice.c
**
**	Dato un grafo orientato G=(V,E) lo si rappresenti mediante
**	una matrice di adiacenza. Dato un vertice v verificare che
**	v abbia in G un unico predecessore. In tal caso costruire
**	il grafo G' ottenuto da G eliminando v e collegando il
**	predecessore di v a tutti i successori di v.
**
*/
//------------------------------------------------------------

/*
**	verifica
**
**	per ogni vertice i di G diverso da v verifica se
**	esiste uno spigolo (i,v) entrante in v e conta il
**	numero di predecessori di v. Se v ha un unico
**	predecessore restituisce il numero di tale vertice,
**	altrimenti restituisce -1.
*/

int verifica(Grafo& g, int v) {
  int i,j, predecessore, npred;
  
  npred = 0;
  predecessore = -1;
  //per contare i predecessori basta guardare le colonne della matrice in posizione i-ma
  
  for (j=0; j<g.n_nodi; j++){      
    for (i=0; i<g.n_nodi; i++) {
      if (i != v && g.nodi[i][j] != MAX_INT) {
	predecessore = i;
	npred++;
      }
    }
  }
  
  if (npred == 1)
    return(predecessore);
  else
    return(-1);
}


/*
**	elimina_vertice
**
**	elimina dal grafo il vertice v, collegando il predecessore
**	 di v a tutti i vertici di G successori di v.
*/

void elimina_vertice(Grafo& g, int predecessore, int v) {
  int i,j;
  
  for (i=0; i<g.n_nodi; i++) {
      if (g.nodi[v][i] != MAX_INT) {
	g.nodi[predecessore][i] = g.nodi[v][i];
      }
  }
  g.nodi[predecessore][v] = MAX_INT;
  
 //return;
}


/*
**	nuovo_grafo;
**
**	costruisce la matrice di adiacenza del grafo G, privato
**	del vertice v.
*/

Grafo& nuovo_grafo(Grafo& vecchioGrafo, int nodoEliminato) {
  Grafo g;
  InitGrafo(g, vecchioGrafo.n_nodi-1);
  for(int i = 0; i < g.n_nodi; i++){
    if(i < nodoEliminato){
      for(int j = 0; j < g.n_nodi; j++){
	if(j<nodoEliminato){
	  g.nodi[i][j] = vecchioGrafo.nodi[i][j];
	}
	else{
	  g.nodi[i][j] = vecchioGrafo.nodi[i][j+1];
	}
      }
    }
    //se l'indice e' maggiore del nodo eliminato
    else{
      for(int j = 0; j < g.n_nodi; j++){
	if(j<nodoEliminato){
	  g.nodi[i][j] = vecchioGrafo.nodi[i+1][j];
	}
	else{
	  g.nodi[i][j] = vecchioGrafo.nodi[i+1][j+1];
	}
      }
    }
  }
  return g;
}


int main() {
	Grafo g;
	
	/*
	InitGrafo(g,10);
	Connect(g,0,1,3);
	Connect(g,0,3,6);
	Connect(g,0,4,4);
	
	Connect(g,1,4,1);
	Connect(g,1,5,3);
	Connect(g,1,2,5);
	
	Connect(g,2,6,2);
	
	Connect(g,3,7,2);
	
	Connect(g,4,3,2);
	Connect(g,4,5,1);
	Connect(g,4,8,3);
	
	Connect(g,5,2,1);
	Connect(g,5,8,5);
	Connect(g,5,9,2);
	
	Connect(g,6,5,3);
	Connect(g,6,9,2);
	
	Connect(g,7,4,1);
	
	Connect(g,8,7,4);
	
	Connect(g,9,8,2);
	*/

	InitGrafo(g, 5);
	Connect(g, 0, 1, 1);
	Connect(g, 0, 2, 2);
	Connect(g, 0, 4, 6);

	Connect(g, 1, 3, 1);
	Connect(g, 1, 4, 3);

	Connect(g, 2, 3, 2);

	Connect(g, 3, 4, 1);

	StampaGrafo(g);
	
// 	cout<<"Inserisci il vertice v: "<<endl;
// 	cin<<v;
// 	
// 	pred = verifica(&m[0][0], n, v);
// 	
// 	if (pred >= 0) {
// 	  elimina_vertice(pred, v, &m[0][0], n);
// 	  nuovo_grafo(&m[0][0], &lista[0], n);
// 	  stampa_grafo(&lista[0], n);
// 	} else {
// 	  printf("Il predecessore di %d non e` unico.\n");
// 	}
	
	//VisitaAmpiezza(g,0);
	//VisitaProfondita(g,0);
	//VisitaProfonditaRicor(g,0, NULL);
	//system("pause");
	return 0;
}
